<?php
/*******************************************************************
 *			
 * 			Fichier			:	functions.php 
 * 			Cr le			:	27 mars 2002 
 * 			Dernire modif	:	01 juillet 2003 
 * 			Email			:	wascripts@phpcodeur.net 
 * 
 * 				Copyright  2002-2003 phpCodeur
 * 
 *******************************************************************/

/*******************************************************************
 *  This program is free software; you can redistribute it and/or 
 * 	modify it under the terms of the GNU General Public License as 
 * 	published by the Free Software Foundation; either version 2 of 
 *  the License, or (at your option) any later version. 
 *******************************************************************/


if( !defined('FUNCTIONS_INC') )
{

define('FUNCTIONS_INC', true); 

/*
 * generate_key()
 * 
 * Gnration d'une chane alatoire
 * 
 * @param integer $num_char	: nombre de caractres
 * 
 * @return string
 */
function generate_key($num_char = 32)
{
	$rand_str = md5( uniqid( rand() ) ); 
	
	return ( $num_char >= 32 ) ? $rand_str : substr($rand_str, 0, $num_char); 
}

/*
 * make_script_url()
 * 
 * Construction de l'url du script
 * 
 * @param string $url	: Url relative
 * 
 * @return string
 */
function make_script_url($url = '')
{
	global $nl_config; 
	
	$excluded_ports = array(80, 8080); 
	
	$server_name = preg_replace('/^http(s)?:\/\/(.*?)\/?$/', 'http\\1://\\2', $nl_config['urlsite']); 
	$server_port = ( !in_array(getenv('SERVER_PORT'), $excluded_ports) ) ? ':' . getenv('SERVER_PORT') : ''; 
	$script_path = ( $nl_config['path'] != '/' ) ? preg_replace('/^\/?(.*?)\/?$/', '/\\1/', $nl_config['path']) : '/'; 
	
	return $server_name . $server_port . $script_path . $url; 
}

/*
 * Location()
 * 
 * Fonction de redirection du script avec url absolue, d'aprs les 
 * spcifications HTTP/1.1
 * 
 * @param string $url	: Url relative de redirection
 * 
 * @return void
 */
function Location($url)
{
	global $nl_config, $db; 
	
	//
	// On ferme la connexion  la base de donnes, si elle existe 
	//
	if( isset($db) && is_object($db) )
	{
		$db->close_connexion(); 
	}
	
	$use_refresh = preg_match("#Microsoft|WebSTAR|Xitami#i", getenv('SERVER_SOFTWARE')); 
	$header_location = ( $use_refresh ) ? 'Refresh: 0; URL=' : 'Location: '; 
	
	$absolute_url  = make_script_url() . ( ( defined('IN_ADMIN') ) ? 'admin/' : '' ); 
	$absolute_url .= str_replace('&amp;', '&', $url); 
	
	header($header_location . $absolute_url); 
	
	//
	// Si la fonction header() ne donne rien, on affiche une page de redirection 
	//
	$message = 'Your browser doesn\'t support meta redirect, click <a href="' . $url . '">here</a> to go on next page.'; 
	display_html_page('Redirection', $message, meta_refresh($url, 0)); 
	
	exit;
}

/*
 * display_html_page()
 * 
 * Gnration d'une apge html basique
 * 
 * @param string $title				: Titre de la page
 * @param string $body				: Corps de la page
 * @param string $additional_meta	: Texte additionnel  ajouter dans l'entte de la page
 * 
 * @return void
 */
function display_html_page($title, $body, $additional_meta = '')
{
	echo "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"; 
	echo "<html>\n<head>\n<title>$title</title>\n"; 
	echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\" />"; 
	echo $additional_meta . "\n</head>\n<body>\n\n<br />"; 
	echo $body; 
	echo "<br />\n\n</body>\n</html>\n"; 
}

/*
 * meta_refresh()
 * 
 * Fonction qui construit le meta "Refresh", ceci afin d'viter 
 * d'avoir du html se baladant dans le code php
 * 
 * @param string $url	: Url de redirection
 * @param string $timer	: Temps avant redirection
 * 
 * @return string
 */
function meta_refresh($url, $timer)
{
	return '<meta http-equiv="Refresh" content="' . $timer . ';url=' . $url . '" />'; 
}

/*
 * mozilla_nav()
 * 
 * Construction de la barre de navigation Mozilla
 * 
 * @param string $rel	: Type de 'relation' du lien
 * @param string $url	: Url de la page
 * @param string $title	: Libell du lien
 * 
 * @return void
 */
function mozilla_nav($rel = '', $url = '', $title = '')
{
	global $s_nav_links; 
	
	if( $rel != '' && $url != '' && $title != '' && !ereg('\.html$', $url) )
	{
		$s_nav_links .= "<link rel=\"$rel\" href=\"" . sessid($url) . "\" title=\"$title\" />\n"; 
	}
}

/*
 * get_data()
 * 
 * Rcupration du nombre de logs, inscrits, inscrits en attente 
 * de confirmation pour la/les liste(s) donne(s)
 * 
 * @param mixed $liste_id_mixed	: Id, ou tableau contenant les id de la/les listes 
 * 
 * @return array
 */
function get_data($liste_id_mixed)
{
	global $db; 
	
	if( is_array($liste_id_mixed) )
	{
		$sql_where = 'IN(' . implode(', ', $liste_id_mixed) . ')'; 
	}
	else
	{
		$sql_where = '= ' . $liste_id_mixed; 
	}
	
	$data = array('num_inscrits' => 0, 'num_temp' => 0, 'num_logs' => 0, 'last_log' => 0); 
	
	$sql = "SELECT COUNT(DISTINCT(a.abo_id)) AS num_abo, a.abo_status 
		FROM " . ABONNES_TABLE . " AS a, " . ABO_LISTE_TABLE . " AS al 
		WHERE al.liste_id $sql_where 
			AND a.abo_id = al.abo_id 
		GROUP BY a.abo_status"; 
	if( !($result = $db->query($sql)) )
	{
		trigger_error('Impossible d\'obtenir le nombre d\'inscrits/inscrits en attente', ERROR); 
	}
	
	while( $row = $db->fetch_array($result) )
	{
		if( $row['abo_status'] == ABO_ACTIF )
		{
			$data['num_inscrits'] = $row['num_abo']; 
		}
		else
		{
			$data['num_temp'] = $row['num_abo']; 
		}
	}
	
	$sql = "SELECT SUM(liste_numlogs) AS num_logs 
		FROM " . LISTE_TABLE . " 
		WHERE liste_id " . $sql_where; 
	if( !($result = $db->query($sql)) )
	{
		trigger_error('Impossible d\'obtenir le nombre de logs envoys', ERROR); 
	}
	
	if( $row = $db->fetch_array($result) )
	{
		$data['num_logs'] = $row['num_logs']; 
	}
	
	$sql = "SELECT MAX(log_date) AS last_log 
		FROM " . LOG_TABLE . " 
		WHERE log_status = " . STATUS_SENDED . " AND liste_id " . $sql_where; 
	if( !($result = $db->query($sql)) )
	{
		trigger_error('Impossible d\'obtenir la date du dernier envoy', ERROR); 
	}
	
	if( $row = $db->fetch_array($result) )
	{
		$data['last_log'] = $row['last_log']; 
	}
	
	return $data; 
}

/*
 * load_settings()
 * 
 * Initialisation des prfrences et du moteur de templates
 * 
 * @param array $admindata	: Donnes utilisateur
 * 
 * @return void
 */
function load_settings($admindata = array())
{
	global $waroot, $nl_config, $db, $lang, $datetime, $template; 
	
	$template_path = $waroot . 'templates/'; 
	if( defined('IN_ADMIN') )
	{
		$template_path .= 'admin/'; 
	}
	$template = new Template($template_path); 
	
	if( !is_array($admindata) )
	{
		$admindata = array(); 
	}
	
	if( !empty($admindata['admin_lang']) )
	{
		$nl_config['language'] = $admindata['admin_lang']; 
	}
	
	if( !empty($admindata['admin_dateformat']) )
	{
		$nl_config['date_format'] = $admindata['admin_dateformat']; 
	}
	
	$language_path = wa_realpath($waroot . 'language/lang_' . $nl_config['language'] . '.php'); 
	
	if( !file_exists($language_path) )
	{
		$nl_config['language'] = 'francais'; 
		$language_path = wa_realpath($waroot . 'language/lang_' . $nl_config['language'] . '.php'); 
		
		if( !file_exists($language_path) )
		{
			trigger_error('<b>Les fichiers de language sont introuvables !</b>', CRITICAL_ERROR); 
		}
	}
	
	include($language_path); 
}

/*
 * wanewsletter_handler()
 * 
 * Gestionnaire d'erreur personnalis du script 
 * 
 * @param integer $errno	: Code de l'erreur
 * @param string  $errstr	: Texte proprement dit de l'erreur
 * @param string  $errfile	: Fichier o s'est produit l'erreur
 * @param integer $errline  : Numro de la ligne 
 * 
 * @return void
 */
function wanewsletter_handler($errno, $errstr, $errfile, $errline)
{
	global $waroot, $nl_config, $db, $admindata, $auth, $template; 
	global $meta, $lang, $s_nav_links, $simple_header, $message, $starttime; 
	
	$debug_text = ''; 
	
	if( defined('IN_CRON') && $errno == ERROR )
	{
		$errno = CRITICAL_ERROR; 
	}
	
	if( ( $errno == CRITICAL_ERROR || $errno == ERROR ) && ( defined('IN_ADMIN') || defined('IN_CRON') || DEBUG_MODE ) )
	{
		if( !empty($db->sql_error['message']) )
		{
			$debug_text .= '<b>SQL query</b> :<br /> ' . nl2br($db->sql_error['query']) . "<br /><br />\n"; 
			$debug_text .= '<b>SQL errno</b> : ' . $db->sql_error['errno'] . "<br />\n"; 
			$debug_text .= '<b>SQL error</b> : ' . $db->sql_error['message'] . "<br />\n<br />\n"; 
		}
		
		$debug_text .= '<b>Fichier</b> : ' . basename($errfile) . " \n<b>Ligne</b> : " . $errline . ' <br />'; 
	}
	
	if( !empty($lang['Message'][$errstr]) )
	{
		$errstr = nl2br($lang['Message'][$errstr]); 
	}
	
	if( $debug_text != '' )
	{
		$errstr .= "<br /><br />\n\n" . $debug_text; 
	}
	
	switch( $errno )
	{
		case CRITICAL_ERROR:
			$message = "<h2>Erreur critique !</h2>\n\n" . $errstr; 
			display_html_page('Erreur critique !', $message); 
			
			exit;
			break;
		
		case ERROR:			
		case MESSAGE:
			if( defined('IN_CRON') )
			{
				exit($errstr); 
			}
			
			if( !defined('IN_WA_FORM') && !defined('IN_SUBSCRIBE') )
			{
				if( $errno == ERROR )
				{
					$msg_title = '<span style="color: #DD3333;">' . $lang['Title']['error'] . '</span>'; 
				}
				else if( $errno == MESSAGE )
				{
					$msg_title = '<span style="color: #33DD33;">' . $lang['Title']['info'] . '</span>'; 
				}
				
				if( !defined('HEADER_INC') )
				{
					include('./page_header.php');
				}
				
				$template->set_filenames(array(
					'body' => 'message_body.tpl'
				));
				
				$template->assign_vars( array(
					'MSG_TITLE' => $msg_title,
					'MSG_TEXT'  => $errstr
				));
				
				$template->pparse('body');
				
				include('./page_footer.php'); 
			}
			
			$message = $errstr; 
			break;
	}
	
	//
	// Dans le cas d'une fonction prcde par @, error_reporting() 
	// retournera 0, dans ce cas, pas d'affichage d'erreur
	//
	$display_error = error_reporting(E_ALL); 
	
	if( $display_error && $errno != ERROR && DEBUG_MODE > 1 )
	{
		if( $errno == E_WARNING )
		{
			echo '<b>Warning !</b> : '; 
		}
		else if( $errno == E_NOTICE )
		{
			echo '<b>Notice</b> : '; 
		}
		else
		{
			exit;
		}
		
		echo $errstr . ' in <b>' . basename($errfile) . '</b> on line <b>' . $errline . '</b></b><br />'; 
	}
}

/*
 * navigation()
 * 
 * Fonction d'affichage par page.
 * 
 * @param string  $url				: adresse vers laquelle doivent pointer les liens de navigation
 * @param integer $total_item		: nombre total d'lments
 * @param integer $item_per_page	: nombre d'lments par page
 * @param integer $page_id			: identifiant de la page en cours
 * 
 * @return string
 */
function navigation($url, $total_item, $item_per_page, $page_id)
{
	global $lang;
	
	$total_pages = ceil($total_item / $item_per_page); 
	
	// premier caractre de l'url au moins en position 1 
	// on place un espace  la position 0 de la chane
	$url = ' ' . $url; 
	
	$url .= ( strpos($url, '?') ) ? '&amp;' : '?'; 
	
	// suppression de l'espace prcdemment ajout 
	$url = substr($url, 1); 
	
	if( $total_pages == 1 )
	{
		return '&nbsp;';
	}
	
	$nav_string = ''; 
	
	if( $total_pages > 10 )
	{
		if( $page_id > 10 )
		{
			$prev = $page_id; 
			do
			{
				$prev--; 
			}
			while( $prev % 10 ); 
			
			$nav_string .= '<a href="' . $url . 'page=1">' . $lang['Start'] . '</a>&nbsp;&nbsp;'; 
			$nav_string .= '<a href="' . $url . 'page=' . $prev . '">' . $lang['Prev'] . '</a>&nbsp;&nbsp;'; 
		}
		
		$current = $page_id; 
		do
		{
			$current--; 
		}
		while( $current % 10 ); 
		
		$current++; 
		
		for( $i = $current; $i < ($current + 10); $i++ )
		{
			if( $i <= $total_pages )
			{
				if( $i > $current )
				{
					$nav_string .= ', '; 
				}
				
				$nav_string .= ( $i == $page_id ) ? '<span style="font-weight: bold;">' . $i . '</span>' : '<a href="' . $url . 'page=' . $i . '">' . $i . '</a>'; 
			}
		}
		
		$next = $page_id; 
		while( $next % 10 )
		{
			$next++; 
		}
		$next++; 
		
		if( $total_pages >= $next )
		{
			$nav_string .= '&nbsp;&nbsp;<a href="' . $url . 'page=' . $next . '">' . $lang['Next'] . '</a>'; 
			$nav_string .= '&nbsp;&nbsp;<a href="' . $url . 'page=' . $total_pages . '">' . $lang['End'] . '</a>'; 
		}
	}
	else
	{
		for( $i = 1; $i <= $total_pages; $i++ )
		{
			if( $i > 1 )
			{
				$nav_string .= ', '; 
			}
			
			$nav_string .= ( $i == $page_id ) ? '<span style="font-weight: bold;">' . $i . '</span>' : '<a href="' . $url . 'page=' . $i . '">' . $i . '</a>'; 
			
		}
	}
	
	return $nav_string; 
}

/*
 * convert_time()
 * 
 * Fonction de renvoi de date selon la langue
 * 
 * @param string  $dateformat	: Format demand
 * @param integer $timestamp	: timestamp unix  convertir
 * 
 * @return string
 */
function convert_time($dateformat, $timestamp)
{
	static $orig_ary, $repl_ary; 
	
	if( !isset($orig_ary) || !isset($repl_ary) )
	{
		global $datetime; 
		
		$orig_ary = $repl_ary = array(); 
		
		foreach( $datetime AS $orig_word => $repl_word )
		{
			$orig_ary[] = '/\b' . $orig_word . '\b/i'; 
			$repl_ary[] = $repl_word; 
		}
	}
	
	return preg_replace($orig_ary, $repl_ary, date($dateformat, $timestamp)); 
}

/*
 * build_listbox()
 * 
 * Selon le mode, affiche une page de slection d'une liste, insert la 
 * liste droulante de changement de liste dans le coin en bas  droite de la page, 
 * ou retourne le code html de la liste droulante de slection
 * 
 * @param string  $mode			: Mode de fonctionnement  utiliser
 * @param integer $auth_type	: Code de permission
 * @param string  $jump_to		: Url de destination de l'ventuel formulaire (si mode display ou corner)
 * 
 * Si mode diffrent de display et de corner, la fonction retourne le code html de 
 * la liste droulante, dans ce cas, la fonction rcupre des paramtres supplmentaires avec func_get_args()
 * 
 * @param integer $not_this_id	: Id de liste  ignorer
 * @param boolean $confirm_only	: Si on ne veut que les listes qui demandent confirmation
 * 
 * @return string
 */
function build_listbox($mode, $auth_type, $jump_to = '')
{
	global $waroot, $db, $nl_config, $admindata, $auth, $session, $template; 
	global $meta, $lang, $s_nav_links, $simple_header, $starttime; 
	
	$list_box = false; 
	$tmp_box  = ''; 
	$args_ary = func_get_args(); 
	
	if( $mode == 'display' || $mode == 'corner' )
	{
		$jump_to = $args_ary[2]; 
		$select_name = 'liste'; 
	}
	else
	{
		$select_name  = ( !empty($args_ary[2]) ) ? $args_ary[2] : 'liste'; 
		$not_this_id  = ( !empty($args_ary[3]) ) ? $args_ary[3] : 0; 
		$confirm_only = ( !empty($args_ary[4]) ) ? $args_ary[4] : 0; 
	}
	
	$liste_id_ary = $auth->check_auth($auth_type); 
	
	foreach( $auth->listdata AS $liste_id => $data )
	{
		if( in_array($liste_id, $liste_id_ary) && ( $mode == 'display' || $mode == 'corner' || ( $liste_id != $not_this_id && ( !$confirm_only || $data['confirm_subscribe'] ) ) ) )
		{
			$selected = ( $admindata['session_liste'] == $liste_id ) ? ' selected="selected"' : ''; 
			
			$liste_name = $data['liste_name']; 
			if( strlen($liste_name) > 30 )
			{
				$liste_name = substr($liste_name, 0, 30); 
				$liste_name = substr($liste_name, 0, strrpos($liste_name, ' ')) . '...'; 
			}
			
			$tmp_box .= '<option value="' . $liste_id . '"' . $selected . '> - ' . $liste_name . ' - </option>'; 
		}			
	}
	
	if( $tmp_box != '' )
	{
		$list_box  = '<select name="' . $select_name . '"'; 
		if( $mode == 'corner' )
		{
			$no_this_list = ''; 
			if( $admindata['session_liste'] > 0 )
			{
				$no_this_list = '&& this.options[this.selectedIndex].value != ' . $admindata['session_liste']; 
			}
			
			$list_box .= ' onchange="if( this.options[this.selectedIndex].value != 0 ' . $no_this_list . ' ) { this.form.submit(); }">'; 
			$list_box .= '<option value="0"> - ' . $lang['Choice_liste'] . ' - </option>'; 
			$list_box .= '<option value="0"> -------------------- </option>'; 
		}
		else
		{
			$list_box .= '>'; 
		}
		
		$list_box .= $tmp_box . '</select>'; 
		
		if( $mode == 'corner' )
		{
			$template->set_filenames(array(
				'list_box_body' => 'list_box.tpl'
			)); 
			
			$template->assign_vars(array(
				'L_VIEW_LIST' 		=> $lang['View_liste'],
				'L_BUTTON_GO' 		=> $lang['Button']['go'],
				
				'S_LISTBOX' 		=> $list_box,
				'S_HIDDEN_FIELDS' 	=> '<input type="hidden" name="sessid" value="' . $session->session_id . '" />',
				
				'U_LISTBOX'			=> sessid($jump_to)
			)); 
			
			$template->assign_var_from_handle('LISTBOX', 'list_box_body'); 
		}
		else if( $mode == 'display' )
		{
			include('./page_header.php'); 
			
			$template->set_filenames(array(
				'body' => 'select_liste_body.tpl'
			)); 
			
			$template->assign_vars(array(
				'L_TITLE'			=> $lang['Title']['select'],
				'L_SELECT_LISTE'	=> $lang['Choice_liste'],
				'L_VALID_BUTTON'	=> $lang['Button']['valid'],
				
				'LISTE_BOX'			=> $list_box,
				'S_HIDDEN_FIELDS'	=> '<input type="hidden" name="sessid" value="' . $session->session_id . '" />',
				'U_FORM'			=> sessid($jump_to)
			)); 
			
			$template->pparse('body'); 
			
			include('./page_footer.php'); 
		}
	}
	else if( $mode == 'display' )
	{
		$message = $lang['Message']['No_liste_exists']; 
		
		if( $admindata['admin_level'] == ADMIN )
		{
			$message .= '<br /><br />' . sprintf($lang['Click_create_liste'], '<a href="' . sessid('view.php?mode=liste&amp;action=add') . '">', '</a>'); 
		}
		
		trigger_error($message, MESSAGE); 
	}
	
	return $list_box; 
}

/*
 * files_list()
 * 
 * Affichage des fichiers joints 
 * 
 * @param string  $page			: Page concerne (pour les liens) (envoi.php ou view.php)
 * @param array   $logdata		: Donnes du log concern
 * @param integer $num_files	: Nombre de fichiers
 * @param integer $format		: Format du log visualis (si dans view.php)
 * 
 * @return boolean
 */
function files_list($page, $logdata, $num_files, $format = 0)
{
	global $waroot, $template, $lang; 
	
	$total_log_size  = 1024; // ~ 1024 correspond au poids de base d'un email (enttes) 
	
	$body_size = (strlen($logdata['log_body_text']) + strlen($logdata['log_body_html'])); 
	
	$total_log_size += ( $body_size > 0 ) ? ($body_size / 2) : 0; 
	
	$test_ary = array(); 
	for( $i = 0; $i < $num_files; $i++ )
	{
		$total_log_size += $logdata['joined_files'][$i]['file_size']; 
		
		$test_ary[] = $logdata['joined_files'][$i]['file_real_name']; 
	}
	
	if( $format == FORMAT_HTML )
	{
		preg_match_all('/<.+?"cid:([^\\:*\/?<">|]+)"[^>]*>/i', $logdata['log_body_html'], $matches); 
		
		$embed_ary = array_intersect($test_ary, $matches[1]); 
		
		if( ($num_files - count($embed_ary)) == 0 )
		{
			return false;
		}
	}
	
	if( $total_log_size >= 1048576 )
	{
		$lang_size 		 = $lang['MO']; 
		$total_log_size /= 1048576; 
	}
	else if( $total_log_size > 1024 )
	{
		$lang_size 		 = $lang['KO']; 
		$total_log_size /= 1024; 
	}
	else
	{
		$lang_size = $lang['Octets']; 
	}
	
	$l_total_log_size = sprintf('%.2f ' . $lang_size, $total_log_size); 
	
	$template->set_filenames(array(
		'files_box_body' => 'files_box.tpl'
	)); 
	
	$template->assign_vars(array(
		'L_FILENAME' 		=> $lang['Filename'],
		'L_FILESIZE' 		=> $lang['Filesize'],
		'L_DEL_FILE_BUTTON'	=> $lang['Button']['del_file'],
		'L_TOTAL_LOG_SIZE'	=> $lang['Total_log_size'],
		
		'TOTAL_LOG_SIZE'	=> $l_total_log_size
	)); 
	
	if( $page == 'envoi.php' )
	{
		$template->assign_block_vars('del_column', array()); 
	}
	
	$u_download = $page . '?mode=download&amp;fid='; 
	
	for( $i = 0; $i < $num_files; $i++ )
	{
		$filesize = $logdata['joined_files'][$i]['file_size']; 
		$filename = $logdata['joined_files'][$i]['file_real_name']; 
		$file_id  = $logdata['joined_files'][$i]['file_id']; 
		$mimetype = $logdata['joined_files'][$i]['file_mimetype']; 
		
		if( $format == FORMAT_HTML && in_array($filename, $embed_ary) )
		{
			continue;
		}
		
		if( $filesize >= 1048576 )
		{
			$lang_size = $lang['MO']; 
			$filesize /= 1048576; 
		}
		else if( $filesize > 1024 )
		{
			$lang_size = $lang['KO']; 
			$filesize /= 1024; 
		}
		else
		{
			$lang_size = $lang['Octets']; 
		}
		
		$l_filesize = sprintf('%.2f ' . $lang_size, $filesize); 
		
		if( ereg('^image/(gif|jpg|jpeg|bmp|pjpeg|png)$', $mimetype) )
		{
			$s_show  = '<a href="javascript:show(' . $file_id . ')">'; 
			$s_show .= '<img src="' . $waroot . 'images/icon_loupe.gif" border="0" alt="' . $lang['Show'] . '" title="' . $lang['Show'] . '" /></a>'; 
			$s_show .= '</a>'; 
		}
		else
		{
			$s_show = ''; 
		}
		
		$template->assign_block_vars('file_info', array(
			'OFFSET'	 => ($i + 1),
			'FILENAME'	 => htmlspecialchars($filename),
			'FILESIZE'	 => $l_filesize,
			'S_SHOW'	 => $s_show,
			'U_DOWNLOAD' => sessid($u_download . $file_id)
		)); 
		
		if( $page == 'envoi.php' )
		{
			$template->assign_block_vars('file_info.delete_options', array(
				'FILE_ID' => $file_id
			)); 
		}
	}
	
	$template->assign_var_from_handle('JOINED_FILES_BOX', 'files_box_body'); 
	
	return true; 
}

/*
 * error_box()
 * 
 * Affichage des erreurs survenues aprs soumission d'un formulaire
 * 
 * @param mixed $msg_error_mixed	: L'erreur, ou les erreurs survenues
 * 
 * @return void
 */
function error_box($msg_error_mixed)
{
	global $template; 
	
	$msg_error = '<ul type="square" class="mediumtexte"><li> '; 
	if( is_array($msg_error_mixed) )
	{
		$msg_error .= implode(' </li><li> ', $msg_error_mixed); 
	}
	else
	{
		$msg_error .= $msg_error_mixed; 
	}
	$msg_error .= ' </li></ul>'; 
	
	$template->set_filenames( array(
		'error_box_body' => 'error_box.tpl'
	));
	
	$template->assign_vars( array(
		'MSG_ERROR' => $msg_error
	));
	
	$template->assign_var_from_handle('ERROR_BOX', 'error_box_body'); 
}

/*
 * purge_liste()
 * 
 * Fonction de purge de la table des abonns 
 * Retourne le nombre d'entres supprimes
 * Fonction rcursive
 * 
 * @param integer $liste_id			: Liste concerne
 * @param integer $limitevalidate	: Limite de validit pour confirmer une inscription
 * @param integer $purge_freq		: Frquence des purges
 * 
 * @return integer
 */
function purge_liste($liste_id = 0, $limitevalidate = 0, $purge_freq = 0)
{
	global $db, $nl_config; 
	
	if( !$liste_id )
	{
		$total_entries_deleted = 0; 
		
		$sql = "SELECT liste_id, limitevalidate, purge_freq 
			FROM " . LISTE_TABLE . " 
			WHERE purge_next < " . time() . " 
				AND auto_purge = " . TRUE; 
		if( !($result = $db->query($sql)) )
		{
			trigger_error('Impossible d\'obtenir les listes de diffusion  purger', ERROR); 
		}
		
		while( $row = $db->fetch_array($result) )
		{
			$total_entries_deleted += purge_liste($row['liste_id'], $row['limitevalidate'], $row['purge_freq']); 
		}
		
		//
		// Optimisation des tables
		//
		$db->check(array(ABONNES_TABLE, ABO_LISTE_TABLE)); 
		
		return $total_entries_deleted; 
	}
	else
	{
		$sql = "SELECT a.abo_id 
			FROM " . ABONNES_TABLE . " AS a, " . ABO_LISTE_TABLE . " AS al 
			WHERE al.liste_id = $liste_id 
				AND a.abo_id = al.abo_id 
				AND a.abo_status = " . ABO_INACTIF . " 
				AND a.abo_register_date < " . (time() - ($limitevalidate * 86400)); 
		if( !($result = $db->query($sql)) )
		{
			trigger_error('Impossible d\'obtenir les entres  supprimer', ERROR); 
		}
		
		$abo_id_ary = array(); 
		while( $row = $db->fetch_array($result) )
		{
			$abo_id_ary[] = $row['abo_id']; 
		}
		
		if( $num_abo_deleted = count($abo_id_ary) )
		{
			$db->transaction(START_TRC); 
			
			$sql = "DELETE FROM " . ABO_LISTE_TABLE . " 
				WHERE abo_id IN(" . implode(', ', $abo_id_ary) . ")"; 
			if( !$db->query($sql) )
			{
				trigger_error('Impossible de supprimer les entres primes de la table abo_liste', ERROR); 
			}
			
			$sql = "DELETE FROM " . ABONNES_TABLE . " 
				WHERE abo_id IN(" . implode(', ', $abo_id_ary) . ")"; 
			if( !$db->query($sql) )
			{
				trigger_error('Impossible de supprimer les entres primes de la table abonnes', ERROR); 
			}
			
			$sql = "UPDATE " . LISTE_TABLE . " 
				SET purge_next = " . (time() + ($purge_freq * 86400)) . " 
				WHERE liste_id = " . $liste_id; 
			if( !$db->query($sql) )
			{
				trigger_error('Impossible de mettre  jour la table liste', ERROR); 
			}
			
			$db->transaction(END_TRC); 
		}
		
		return $num_abo_deleted; 
	}
}

/*
 * emul_magic_quotes_gpc()
 * 
 * Simulation de la configuration magic_quotes_gpc  on
 * Fonction rcursive
 * 
 * @param array $data	: Tableau des donnes
 * 
 * @return void
 */
function emul_magic_quotes_gpc(&$data)
{
	if( is_array($data) )
	{
		foreach( $data AS $key => $val )
		{
			if( is_array($val) )
			{
				emul_magic_quotes_gpc($val); 
			}
			else
			{
				$data[$key] = ( is_numeric($val) ) ? $val : addslashes($val); 
			}
		}
	}
}

/*
 * wa_realpath()
 * 
 * Fonction similaire  realpath() mais utilise un autre moyen d'obtenir l'url canonique si 
 * la fonction realpath() est dsactive
 * 
 * @param string $relative_path	: Url relative  rsoudre
 * 
 * @return string
 */
function wa_realpath($relative_path)
{
	global $waroot; 
	
	if( @function_exists('realpath') && @realpath($waroot . 'includes/functions.php') )
	{
		return str_replace('\\', '/', realpath($relative_path)); 
	}
	else
	{
		$script_path = ( !empty($_SERVER['SCRIPT_FILENAME']) ) ? dirname($_SERVER['SCRIPT_FILENAME']) : ''; 
		
		//
		// FIX multimania
		//
		if( stristr($script_path, 'free') && stristr($script_path, 'multimania') )
		{
			return $relative_path; 
		}
		
		if( $script_path != '' )
		{
			$script_path = str_replace('\\', '/', $script_path); 
			
			$ary = explode('/', $relative_path); 
			$dir = array(); 
			
			for( $i = 0; $i < count($ary); $i++ )
			{
				if( $ary[$i] != '..' && $ary[$i] != '.' )
				{
					$dir[] = $ary[$i]; 
				}
				else if( $ary[$i] == '..' )
				{
					$script_path = substr($script_path, 0, strrpos($script_path, '/')); 
				}
			}
			
			return $script_path . '/' . implode('/', $dir); 
		}
	}
	
	return $relative_path; 
}

/*
 * unhtmlspecialchars()
 * 
 * Fonction inverse de la fonction htmlspecialchars()
 * 
 * @param string $input
 * 
 * @return string
 */
function unhtmlspecialchars($input)
{
	$html_entities = array('/&lt;/', '/&gt;/', '/&quot;/', '/&amp;/'); 
	$html_replace  = array('<', '>', '"', '&'); 
	
	return preg_replace($html_entities, $html_replace, $input); 
}

/*
 * active_urls()
 * 
 * Convertit les liens dans un texte en lien html
 * Import de WAgoldBook 2.0.x
 * 
 * @param string $message
 * 
 * @return string
 */
function active_urls($message)
{
	$message = ' ' . $message; 
	
	$message = preg_replace("#([\n ])([a-z]+?)://([^,\t \n\r]+)#i", "\\1<a href=\"\\2://\\3\" target=\"_blank\">\\2://\\3</a>", $message);
	$message = preg_replace("#([\n ])www\.([a-z0-9\-]+)\.([a-z0-9\-.\~]+)((?:/[^,\t \n\r]*)?)#i", "\\1<a href=\"http://www.\\2.\\3\\4\" target=\"_blank\">www.\\2.\\3\\4</a>", $message);
	$message = preg_replace("#([\n ])([a-z0-9\-_.]+?)@([\w\-]+\.([\w\-\.]+\.)?[\w]+)#i", "\\1<a href=\"mailto:\\2@\\3\">\\2@\\3</a>", $message);
	
	// Remove our padding..
	$message = substr($message, 1); 
	
	return $message;
}

/*
 * fake_header()
 * 
 * Fonctions  utiliser lors des longues boucles (backup, envois) 
 * qui peuvent provoquer un time out du navigateur client 
 * Inspir d'un code quivalent dans phpMyAdmin 2.5.0 (libraries/build_dump.lib.php prcisment)
 * 
 * @param boolean $in_loop		: True si on est dans la boucle, false pour initialiser $time
 * 
 * @return void
 */
function fake_header($in_loop)
{
	static $time; 
	
	if( $in_loop )
	{
		$new_time = time(); 
		
		if( ($new_time - $time) >= 30 )
		{
			$time = $new_time; 
			header('X-WaPing: Pong'); 
		}
	}
	else
	{
		$time = time(); 
	}
}

/*
 * make_sql_ary()
 * 
 * Parse un fichier contenant une liste de requte et 
 * renvoie un tableau avec une requte par entre
 * 
 * @param string $input 		: Contenu du fichier .sql
 * @param string $delimiter		: Dlimiteur entre chaque requte (en gnral -> ; )
 * @param string $prefixe		: Prfixe des tables  mettre  la place du prefixe par dfaut
 * 
 * @return array
 */
function make_sql_ary($input, $delimiter, $prefixe = '')
{
	$tmp			= ''; 
	$output			= array(); 
	$in_comments	= false; 
	$between_quotes	= false; 
	
	$lines			= preg_split("/(\r\n?)|\n/", $input, -1, PREG_SPLIT_DELIM_CAPTURE); 
	$total_lines	= count($lines); 
	
	fake_header(false); 
	
	for( $i = 0; $i < $total_lines; $i++ )
	{
		if( preg_match("/^(\r\n?)|\n$/", $lines[$i]) )
		{
			if( $between_quotes )
			{
				$tmp .= $lines[$i]; 
			}
			
			continue;
		}
		
		//
		// Si on est pas dans des simples quotes, on vrifie si on entre ds des commentaires
		//
		if( !$between_quotes && !$in_comments && preg_match('/^\/\*/', $lines[$i]) )
		{
			$in_comments = true; 
		}
		
		if( $between_quotes || ( !$in_comments && strlen($lines[$i]) > 0 && $lines[$i]{0} != '#' ) )
		{
			//
			// Nombre de simple quotes non chapps
			//
			$unescaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*'/", $lines[$i], $matches); 
			
			if( ( !$between_quotes && !($unescaped_quotes % 2) ) || ( $between_quotes && ($unescaped_quotes % 2) ) )
			{
				if( preg_match('/' . $delimiter . '$/i', rtrim($lines[$i])) )
				{
					$lines[$i] = ( $tmp != '' ) ? rtrim($lines[$i]) : trim($lines[$i]); 
					$output[]  = $tmp . substr($lines[$i], 0, -(strlen($delimiter))); 
					$tmp = ''; 
				}
				else
				{
					$tmp .= ( $tmp != '' ) ? $lines[$i] : ltrim($lines[$i]); 
				}
				
				$between_quotes = false; 
			}
			else
			{
				$between_quotes = true; 
				$tmp .= ( $tmp != '' ) ? $lines[$i] : ltrim($lines[$i]); 
			}
		}
		
		if( !$between_quotes && $in_comments && preg_match("/\*\/$/", rtrim($lines[$i])) )
		{
			$in_comments = false; 
		}
		
		//
		// Pour tenter de mnager la mmoire 
		//
		unset($lines[$i]); 
		
		fake_header(true); 
	}
	
	if( $prefixe != '' )
	{
		$output = preg_replace('/wa_/', $prefixe, $output); 
	}
	
	//
	// Pour tenter de mnager la mmoire 
	//
	unset($input, $lines); 
	
	return $output; 
}

//
// Appel du gestionnaire d'erreur 
//
@set_error_handler('wanewsletter_handler'); 

}
?>